#ifndef _STUDENTINCOURSE_CPP
#define _STUDENTINCOURSE_CPP

#include <string>
#include <fstream>
#include "StudentInCourse.h"

using namespace std;

StudentInCourse::StudentInCourse( int aIdNum, string aCCode )
{
   if ( aIdNum < 0 )
      throw StudentInCourseXptn( StudentInCourse::SICKey(),
                                 "StudentInCourse",
                                 StudentInCourseXptn::negIdNum,
                                 aIdNum );
   sicSIdNum = aIdNum;
   if ( aCCode.length() != Course::cCodeSz-1 && aCCode.length() != 0 )
      throw StudentInCourseXptn(
                               StudentInCourse::SICKey(sicSIdNum),
                                 "StudentInCourse",
                                 StudentInCourseXptn::keyLen,
                                 aCCode.c_str() );
   sicCCode = Course::CourseKey( aCCode );
   sicMark = 0;
} // StudentInCourse::StudentInCourse

//-----------------------------------------------------------------
//                             
//-----------------------------------------------------------------
void StudentInCourse::setSIdNum( int aIdNum )
{
   if ( aIdNum <= 0 )
      throw StudentInCourseXptn( getKey(), "setSIdNum",
                                 StudentInCourseXptn::negIdNum,
                                 aIdNum );
   sicSIdNum = aIdNum;
} // StudentInCourse::setSIdNum

void StudentInCourse::setCCode( string aCCode )
{
   if ( aCCode.length() != Course::cCodeSz-1 )
      throw StudentInCourseXptn( getKey(), "setCCode",
                                 StudentInCourseXptn::keyLen,
                                 aCCode.c_str() );
   sicCCode = Course::CourseKey( aCCode );
} // StudentInCourse::setCCode

void StudentInCourse::setMark( float aMark )
{
   if ( aMark < 0 || 10 < aMark )
      throw StudentInCourseXptn( getKey(), "setMark",
                                 StudentInCourseXptn::rangeError,
                                 aMark );
   sicMark = aMark;
} // StudentInCourse::setMark

void StudentInCourse::save( ostream& bout ) const
{
   if ( bout.fail() )
      throw StudentInCourseXptn( getKey(), "save",
                               StudentInCourseXptn::fileNotOpen );
   bout.write( reinterpret_cast<const char*>(&sicSIdNum),
               sizeof(sicSIdNum) );             //  
   bout.write( sicCCode.s, Course::cCodeSz ); //  
   bout.write( reinterpret_cast<const char*>(&sicMark),
               sizeof(sicMark) );             //   
   if ( bout.fail() )
      throw StudentInCourseXptn( getKey(), "save",
                               StudentInCourseXptn::cannotWrite );
} // StudentInCourse::save

void StudentInCourse::load( istream& bin )
{
   StudentInCourse tmp;
   bin.read( reinterpret_cast<char*>(&tmp.sicSIdNum),
             sizeof(sicSIdNum) );               //  
   if ( !bin.eof() )
   {
      bin.read( tmp.sicCCode.s, Course::cCodeSz );
                                                 // . 
      bin.read( reinterpret_cast<char*>(&tmp.sicMark),
                sizeof(sicMark) );            //   
      if ( bin.fail() )
         throw StudentInCourseXptn( getKey(), "load",
                               StudentInCourseXptn::cannotWrite );
      *this = tmp;
   }
} // StudentInCourse::load

void StudentInCourse::display( ostream& tout ) const
{
   tout << sicSIdNum << ' ' << sicCCode.s << ' ' << sicMark << endl;
} // StudentInCourse::display

#endif // _STUDENTINCOURSE_CPP
